@@ -41,14 +41,16 @@ namespace cl {
 // ParseCommandLineOptions - Command line option processing entry point.
 //
 void ParseCommandLineOptions(int argc, char **argv,
-                             const char *Overview = 0);
+                             const char *Overview = 0,
+                             bool ReadResponseFiles = false);
 
 //===----------------------------------------------------------------------===//
 // ParseEnvironmentOptions - Environment variable option processing alternate
 //                           entry point.
 //
 void ParseEnvironmentOptions(const char *progName, const char *envvar,
-                             const char *Overview = 0);
+                             const char *Overview = 0,
+                             bool ReadResponseFiles = false);
 
 ///===---------------------------------------------------------------------===//
 /// SetVersionPrinter - Override the default (LLVM specific) version printer
@@ -123,7 +125,8 @@ enum FormattingFlags {
 enum MiscFlags {               // Miscellaneous flags to adjust argument
   CommaSeparated     = 0x200,  // Should this cl::list split between commas?
   PositionalEatsArgs = 0x400,  // Should this positional cl::list eat -args?
-  MiscMask           = 0x600   // Union of the above flags.
+  Sink               = 0x800,  // Should this cl::list eat all unknown options?
+  MiscMask           = 0xE00   // Union of the above flags.
 };
 
 
@@ -145,10 +148,10 @@ class Option {
   virtual enum ValueExpected getValueExpectedFlagDefault() const {
     return ValueOptional;
   }
-  
+
   // Out of line virtual function to provide home for the class.
   virtual void anchor();
-  
+
   int NumOccurrences;     // The number of times specified
   int Flags;              // Flags for the argument
   unsigned Position;      // Position of last occurrence of the option
@@ -212,19 +215,19 @@ public:
   // addArgument - Register this argument with the commandline system.
   //
   void addArgument();
-  
+
   Option *getNextRegisteredOption() const { return NextRegistered; }
 
   // Return the width of the option tag for printing...
-  virtual unsigned getOptionWidth() const = 0;
+  virtual size_t getOptionWidth() const = 0;
 
   // printOptionInfo - Print out information about this option.  The
   // to-be-maintained width is specified.
   //
-  virtual void printOptionInfo(unsigned GlobalWidth) const = 0;
+  virtual void printOptionInfo(size_t GlobalWidth) const = 0;
 
   virtual void getExtraOptionNames(std::vector<const char*> &OptionNames) {}
-  
+
   // addOccurrence - Wrapper around handleOccurrence that enforces Flags
   //
   bool addOccurrence(unsigned pos, const char *ArgName,
@@ -331,14 +334,15 @@ public:
 
   template<class Opt>
   void apply(Opt &O) const {
-    for (unsigned i = 0, e = Values.size(); i != e; ++i)
+    for (unsigned i = 0, e = static_cast<unsigned>(Values.size());
+         i != e; ++i)
       O.getParser().addLiteralOption(Values[i].first, Values[i].second.first,
                                      Values[i].second.second);
   }
 };
 
 template<class DataType>
-ValuesClass<DataType> END_WITH_NULL values(const char *Arg, DataType Val, 
+ValuesClass<DataType> END_WITH_NULL values(const char *Arg, DataType Val,
                                            const char *Desc, ...) {
     va_list ValueArgs;
     va_start(ValueArgs, Desc);
@@ -375,12 +379,12 @@ struct generic_parser_base {
   virtual const char *getDescription(unsigned N) const = 0;
 
   // Return the width of the option tag for printing...
-  virtual unsigned getOptionWidth(const Option &O) const;
+  virtual size_t getOptionWidth(const Option &O) const;
 
   // printOptionInfo - Print out information about this option.  The
   // to-be-maintained width is specified.
   //
-  virtual void printOptionInfo(const Option &O, unsigned GlobalWidth) const;
+  virtual void printOptionInfo(const Option &O, size_t GlobalWidth) const;
 
   void initialize(Option &O) {
     // All of the modifiers for the option have been processed by now, so the
@@ -388,7 +392,7 @@ struct generic_parser_base {
     //
     hasArgStr = O.hasArgStr();
   }
-  
+
   void getExtraOptionNames(std::vector<const char*> &OptionNames) {
     // If there has been no argstr specified, that means that we need to add an
     // argument for every possible option.  This ensures that our options are
@@ -456,7 +460,8 @@ public:
     else
       ArgVal = ArgName;
 
-    for (unsigned i = 0, e = Values.size(); i != e; ++i)
+    for (unsigned i = 0, e = static_cast<unsigned>(Values.size());
+         i != e; ++i)
       if (ArgVal == Values[i].first) {
         V = Values[i].second.first;
         return false;
@@ -499,12 +504,12 @@ struct basic_parser_impl {  // non-template implementation of basic_parser<t>
   void initialize(Option &O) {}
 
   // Return the width of the option tag for printing...
-  unsigned getOptionWidth(const Option &O) const;
+  size_t getOptionWidth(const Option &O) const;
 
   // printOptionInfo - Print out information about this option.  The
   // to-be-maintained width is specified.
   //
-  void printOptionInfo(const Option &O, unsigned GlobalWidth) const;
+  void printOptionInfo(const Option &O, size_t GlobalWidth) const;
 
   // getValueName - Overload in subclass to provide a better default value.
   virtual const char *getValueName() const { return "value"; }
@@ -536,7 +541,7 @@ public:
 
   // getValueName - Do not print =<value> at all.
   virtual const char *getValueName() const { return 0; }
-  
+
   // An out-of-line virtual method to provide a 'home' for this class.
   virtual void anchor();
 };
@@ -550,7 +555,7 @@ template<>
 class parser<boolOrDefault> : public basic_parser<boolOrDefault> {
 public:
   // parse - Return true on error.
-  bool parse(Option &O, const char *ArgName, const std::string &Arg, 
+  bool parse(Option &O, const char *ArgName, const std::string &Arg,
              boolOrDefault &Val);
 
   enum ValueExpected getValueExpectedFlagDefault() const {
@@ -559,7 +564,7 @@ public:
 
   // getValueName - Do not print =<value> at all.
   virtual const char *getValueName() const { return 0; }
-  
+
   // An out-of-line virtual method to provide a 'home' for this class.
   virtual void anchor();
 };
@@ -812,8 +817,8 @@ class opt : public Option,
   }
 
   // Forward printing stuff to the parser...
-  virtual unsigned getOptionWidth() const {return Parser.getOptionWidth(*this);}
-  virtual void printOptionInfo(unsigned GlobalWidth) const {
+  virtual size_t getOptionWidth() const {return Parser.getOptionWidth(*this);}
+  virtual void printOptionInfo(size_t GlobalWidth) const {
     Parser.printOptionInfo(*this, GlobalWidth);
   }
 
@@ -964,7 +969,7 @@ class list : public Option, public list_storage<DataType, Storage> {
   virtual void getExtraOptionNames(std::vector<const char*> &OptionNames) {
     return Parser.getExtraOptionNames(OptionNames);
   }
-  
+
   virtual bool handleOccurrence(unsigned pos, const char *ArgName,
                                 const std::string &Arg) {
     typename ParserClass::parser_data_type Val =
@@ -978,8 +983,8 @@ class list : public Option, public list_storage<DataType, Storage> {
   }
 
   // Forward printing stuff to the parser...
-  virtual unsigned getOptionWidth() const {return Parser.getOptionWidth(*this);}
-  virtual void printOptionInfo(unsigned GlobalWidth) const {
+  virtual size_t getOptionWidth() const {return Parser.getOptionWidth(*this);}
+  virtual void printOptionInfo(size_t GlobalWidth) const {
     Parser.printOptionInfo(*this, GlobalWidth);
   }
 
@@ -1070,7 +1075,7 @@ public:
 template<class DataType, class StorageClass>
 class bits_storage {
   unsigned *Location;   // Where to store the bits...
-  
+
   template<class T>
   static unsigned Bit(const T &V) {
     unsigned BitPos = reinterpret_cast<unsigned>(V);
@@ -1095,9 +1100,9 @@ public:
            "line option with external storage!");
     *Location |= Bit(V);
   }
-  
+
   unsigned getBits() { return *Location; }
-  
+
   template<class T>
   bool isSet(const T &V) {
     return (*Location & Bit(V)) != 0;
@@ -1105,13 +1110,13 @@ public:
 };
 
 
-// Define how to hold bits.  Since we can inherit from a class, we do so. 
+// Define how to hold bits.  Since we can inherit from a class, we do so.
 // This makes us exactly compatible with the bits in all cases that it is used.
 //
 template<class DataType>
 class bits_storage<DataType, bool> {
   unsigned Bits;   // Where to store the bits...
-  
+
   template<class T>
   static unsigned Bit(const T &V) {
     unsigned BitPos = reinterpret_cast<unsigned>(V);
@@ -1119,15 +1124,15 @@ class bits_storage<DataType, bool> {
           "enum exceeds width of bit vector!");
     return 1 << BitPos;
   }
-  
+
 public:
   template<class T>
   void addValue(const T &V) {
     Bits |=  Bit(V);
   }
-  
+
   unsigned getBits() { return Bits; }
-  
+
   template<class T>
   bool isSet(const T &V) {
     return (Bits & Bit(V)) != 0;
@@ -1150,7 +1155,7 @@ class bits : public Option, public bits_storage<DataType, Storage> {
   virtual void getExtraOptionNames(std::vector<const char*> &OptionNames) {
     return Parser.getExtraOptionNames(OptionNames);
   }
-  
+
   virtual bool handleOccurrence(unsigned pos, const char *ArgName,
                                 const std::string &Arg) {
     typename ParserClass::parser_data_type Val =
@@ -1164,8 +1169,8 @@ class bits : public Option, public bits_storage<DataType, Storage> {
   }
 
   // Forward printing stuff to the parser...
-  virtual unsigned getOptionWidth() const {return Parser.getOptionWidth(*this);}
-  virtual void printOptionInfo(unsigned GlobalWidth) const {
+  virtual size_t getOptionWidth() const {return Parser.getOptionWidth(*this);}
+  virtual void printOptionInfo(size_t GlobalWidth) const {
     Parser.printOptionInfo(*this, GlobalWidth);
   }
 
@@ -1257,8 +1262,8 @@ class alias : public Option {
     return AliasFor->handleOccurrence(pos, AliasFor->ArgStr, Arg);
   }
   // Handle printing stuff...
-  virtual unsigned getOptionWidth() const;
-  virtual void printOptionInfo(unsigned GlobalWidth) const;
+  virtual size_t getOptionWidth() const;
+  virtual void printOptionInfo(size_t GlobalWidth) const;
 
   void done() {
     if (!hasArgStr())
@@ -1315,7 +1320,7 @@ struct aliasopt {
 // exit is called.
 struct extrahelp {
   const char * morehelp;
-  extrahelp(const char* help);
+  explicit extrahelp(const char* help);
 };
 
 void PrintVersionMessage();
