@@ -21,7 +21,7 @@
 #include "llvm/ParameterAttributes.h"
 #include "llvm/GlobalVariable.h"
 #include "llvm/Instructions.h"
-#include "llvm/Intrinsics.h"
+#include "llvm/IntrinsicInst.h"
 #include "llvm/Pass.h"
 #include "llvm/Target/TargetData.h"
 #include "llvm/ADT/SmallVector.h"
@@ -83,7 +83,7 @@ namespace {
   // Register this pass...
   char NoAA::ID = 0;
   RegisterPass<NoAA>
-  U("no-aa", "No Alias Analysis (always returns 'may' alias)");
+  U("no-aa", "No Alias Analysis (always returns 'may' alias)", true, true);
 
   // Declare that we implement the AliasAnalysis interface
   RegisterAnalysisGroup<AliasAnalysis> V(U);
@@ -128,7 +128,7 @@ namespace {
   // Register this pass...
   char BasicAliasAnalysis::ID = 0;
   RegisterPass<BasicAliasAnalysis>
-  X("basicaa", "Basic Alias Analysis (default AA impl)");
+  X("basicaa", "Basic Alias Analysis (default AA impl)", false, true);
 
   // Declare that we implement the AliasAnalysis interface
   RegisterAnalysisGroup<AliasAnalysis, true> Y(X);
@@ -138,9 +138,10 @@ ImmutablePass *llvm::createBasicAliasAnalysisPass() {
   return new BasicAliasAnalysis();
 }
 
-// getUnderlyingObject - This traverses the use chain to figure out what object
-// the specified value points to.  If the value points to, or is derived from, a
-// unique object or an argument, return it.
+/// getUnderlyingObject - This traverses the use chain to figure out what object
+/// the specified value points to.  If the value points to, or is derived from,
+/// a unique object or an argument, return it.  This returns:
+///    Arguments, GlobalVariables, Functions, Allocas, Mallocs.
 static const Value *getUnderlyingObject(const Value *V) {
   if (!isa<PointerType>(V->getType())) return 0;
 
@@ -218,8 +219,6 @@ static bool AddressMightEscape(const Value *V) {
         return true;
       break; // next use.
     case Instruction::BitCast:
-      if (!isa<PointerType>(I->getType()))
-        return true;
       if (AddressMightEscape(I))
         return true;
       break; // next use
@@ -227,6 +226,12 @@ static bool AddressMightEscape(const Value *V) {
       // If returned, the address will escape to calling functions, but no
       // callees could modify it.
       break; // next use
+    case Instruction::Call:
+      // If the call is to a few known safe intrinsics, we know that it does
+      // not escape
+      if (!isa<MemIntrinsic>(I))
+        return true;
+      break;  // next use
     default:
       return true;
     }
@@ -241,41 +246,47 @@ static bool AddressMightEscape(const Value *V) {
 //
 AliasAnalysis::ModRefResult
 BasicAliasAnalysis::getModRefInfo(CallSite CS, Value *P, unsigned Size) {
-  if (!isa<Constant>(P))
-    if (const AllocationInst *AI =
-                  dyn_cast_or_null<AllocationInst>(getUnderlyingObject(P))) {
-      // Okay, the pointer is to a stack allocated object.  If we can prove that
-      // the pointer never "escapes", then we know the call cannot clobber it,
-      // because it simply can't get its address.
-      if (!AddressMightEscape(AI))
-        return NoModRef;
+  if (!isa<Constant>(P)) {
+    const Value *Object = getUnderlyingObject(P);
+    // Allocations and byval arguments are "new" objects.
+    if (Object &&
+        (isa<AllocationInst>(Object) || isa<Argument>(Object))) {
+      // Okay, the pointer is to a stack allocated (or effectively so, for 
+      // for noalias parameters) object.  If the address of this object doesn't
+      // escape from this function body to a callee, then we know that no
+      // callees can mod/ref it unless they are actually passed it.
+      if (isa<AllocationInst>(Object) ||
+          cast<Argument>(Object)->hasByValAttr() ||
+          cast<Argument>(Object)->hasNoAliasAttr())
+        if (!AddressMightEscape(Object)) {
+          bool passedAsArg = false;
+          for (CallSite::arg_iterator CI = CS.arg_begin(), CE = CS.arg_end();
+              CI != CE; ++CI)
+            if (isa<PointerType>((*CI)->getType()) &&
+                ( getUnderlyingObject(*CI) == P ||
+                  alias(cast<Value>(CI), ~0U, P, ~0U) != NoAlias) )
+              passedAsArg = true;
+          
+          if (!passedAsArg)
+            return NoModRef;
+        }
 
       // If this is a tail call and P points to a stack location, we know that
       // the tail call cannot access or modify the local stack.
-      if (CallInst *CI = dyn_cast<CallInst>(CS.getInstruction()))
-        if (CI->isTailCall() && isa<AllocaInst>(AI))
-          return NoModRef;
+      // We cannot exclude byval arguments here; these belong to the caller of
+      // the current function not to the current function, and a tail callee
+      // may reference them.
+      if (isa<AllocaInst>(Object))
+        if (CallInst *CI = dyn_cast<CallInst>(CS.getInstruction()))
+          if (CI->isTailCall())
+            return NoModRef;
     }
+  }
 
   // The AliasAnalysis base class has some smarts, lets use them.
   return AliasAnalysis::getModRefInfo(CS, P, Size);
 }
 
-static bool isNoAliasArgument(const Argument *Arg) {
-  const Function *Func = Arg->getParent();
-  const ParamAttrsList *Attr = Func->getParamAttrs();
-  if (Attr) {
-    unsigned Idx = 1;
-    for (Function::const_arg_iterator I = Func->arg_begin(), 
-          E = Func->arg_end(); I != E; ++I, ++Idx) {
-      if (&(*I) == Arg && 
-           Attr->paramHasAttr(Idx, ParamAttr::NoAlias))
-        return true;
-    }
-  }
-  return false;
-}
-
 // alias - Provide a bunch of ad-hoc rules to disambiguate in common cases, such
 // as array references.  Note that this function is heavily tail recursive.
 // Hopefully we have a smart C++ compiler.  :)
@@ -317,9 +328,12 @@ BasicAliasAnalysis::alias(const Value *V1, unsigned V1Size,
         
         // If they are two different objects, and one is a noalias argument
         // then they do not alias.
-        if (O1 != O2 && isNoAliasArgument(O1Arg))
+        if (O1 != O2 && O1Arg->hasNoAliasAttr())
           return NoAlias;
-          
+
+        // Byval arguments can't alias globals or other arguments.
+        if (O1 != O2 && O1Arg->hasByValAttr()) return NoAlias;
+        
         // Otherwise, nothing is known...
       } 
       
@@ -329,16 +343,18 @@ BasicAliasAnalysis::alias(const Value *V1, unsigned V1Size,
         
         // If they are two different objects, and one is a noalias argument
         // then they do not alias.
-        if (O1 != O2 && isNoAliasArgument(O2Arg))
+        if (O1 != O2 && O2Arg->hasNoAliasAttr())
           return NoAlias;
           
+        // Byval arguments can't alias globals or other arguments.
+        if (O1 != O2 && O2Arg->hasByValAttr()) return NoAlias;
+        
         // Otherwise, nothing is known...
       
-      } else if (O1 != O2) {
-        if (!isa<Argument>(O1))
-          // If they are two different objects, and neither is an argument,
-          // we know that we have no alias...
-          return NoAlias;
+      } else if (O1 != O2 && !isa<Argument>(O1)) {
+        // If they are two different objects, and neither is an argument,
+        // we know that we have no alias.
+        return NoAlias;
       }
 
       // If they are the same object, they we can look at the indexes.  If they
@@ -347,9 +363,15 @@ BasicAliasAnalysis::alias(const Value *V1, unsigned V1Size,
       // can't tell anything.
     }
 
-
-    if (!isa<Argument>(O1) && isa<ConstantPointerNull>(V2))
-      return NoAlias;                    // Unique values don't alias null
+    // Unique values don't alias null, except non-byval arguments.
+    if (isa<ConstantPointerNull>(V2)) {
+      if (const Argument *O1Arg = dyn_cast<Argument>(O1)) {
+        if (O1Arg->hasByValAttr()) 
+          return NoAlias;
+      } else {
+        return NoAlias;                    
+      }
+    }
 
     if (isa<GlobalVariable>(O1) ||
         (isa<AllocationInst>(O1) &&
