@@ -21,10 +21,11 @@
 
 #define DEBUG_TYPE "strongphielim"
 #include "llvm/CodeGen/Passes.h"
-#include "llvm/CodeGen/LiveVariables.h"
+#include "llvm/CodeGen/LiveIntervalAnalysis.h"
 #include "llvm/CodeGen/MachineDominators.h"
 #include "llvm/CodeGen/MachineFunctionPass.h"
 #include "llvm/CodeGen/MachineInstr.h"
+#include "llvm/CodeGen/MachineLoopInfo.h"
 #include "llvm/CodeGen/MachineRegisterInfo.h"
 #include "llvm/Target/TargetInstrInfo.h"
 #include "llvm/Target/TargetMachine.h"
@@ -51,9 +52,13 @@ namespace {
     // used as operands to another another PHI node
     std::set<unsigned> UsedByAnother;
     
-    // RenameSets are the sets of operands to a PHI (the defining instruction
-    // of the key) that can be renamed without copies
-    std::map<unsigned, std::set<unsigned> > RenameSets;
+    // RenameSets are the sets of operands (and their VNInfo IDs) to a PHI
+    // (the defining instruction of the key) that can be renamed without copies.
+    std::map<unsigned, std::map<unsigned, unsigned> > RenameSets;
+    
+    // PhiValueNumber holds the ID numbers of the VNs for each phi that we're
+    // eliminating, indexed by the register defined by that phi.
+    std::map<unsigned, unsigned> PhiValueNumber;
 
     // Store the DFS-in number of each block
     DenseMap<MachineBasicBlock*, unsigned> preorder;
@@ -65,7 +70,10 @@ namespace {
     
     virtual void getAnalysisUsage(AnalysisUsage &AU) const {
       AU.addRequired<MachineDominatorTree>();
-      AU.addRequired<LiveVariables>();
+      AU.addRequired<LiveIntervals>();
+      
+      // TODO: Actually make this true.
+      AU.addPreserved<LiveIntervals>();
       MachineFunctionPass::getAnalysisUsage(AU);
     }
     
@@ -121,14 +129,16 @@ namespace {
     void computeDFS(MachineFunction& MF);
     void processBlock(MachineBasicBlock* MBB);
     
-    std::vector<DomForestNode*> computeDomForest(std::set<unsigned>& instrs,
+    std::vector<DomForestNode*> computeDomForest(std::map<unsigned, unsigned>& instrs,
                                                  MachineRegisterInfo& MRI);
     void processPHIUnion(MachineInstr* Inst,
-                         std::set<unsigned>& PHIUnion,
+                         std::map<unsigned, unsigned>& PHIUnion,
                          std::vector<StrongPHIElimination::DomForestNode*>& DF,
                          std::vector<std::pair<unsigned, unsigned> >& locals);
     void ScheduleCopies(MachineBasicBlock* MBB, std::set<unsigned>& pushed);
-    void InsertCopies(MachineBasicBlock* MBB, std::set<MachineBasicBlock*>& v);
+    void InsertCopies(MachineBasicBlock* MBB,
+                      SmallPtrSet<MachineBasicBlock*, 16>& v);
+    void mergeLiveIntervals(unsigned primary, unsigned secondary, unsigned VN);
   };
 
   char StrongPHIElimination::ID = 0;
@@ -164,7 +174,7 @@ void StrongPHIElimination::computeDFS(MachineFunction& MF) {
     }
     
     bool inserted = false;
-    for (MachineDomTreeNode::iterator I = node->begin(), E = node->end();
+    for (MachineDomTreeNode::iterator I = currNode->begin(), E = currNode->end();
          I != E; ++I)
       if (!frontier.count(*I) && !visited.count(*I)) {
         worklist.push_back(*I);
@@ -212,7 +222,7 @@ public:
 /// computeDomForest - compute the subforest of the DomTree corresponding
 /// to the defining blocks of the registers in question
 std::vector<StrongPHIElimination::DomForestNode*>
-StrongPHIElimination::computeDomForest(std::set<unsigned>& regs, 
+StrongPHIElimination::computeDomForest(std::map<unsigned, unsigned>& regs, 
                                        MachineRegisterInfo& MRI) {
   // Begin by creating a virtual root node, since the actual results
   // may well be a forest.  Assume this node has maximum DFS-out number.
@@ -222,9 +232,9 @@ StrongPHIElimination::computeDomForest(std::set<unsigned>& regs,
   // Populate a worklist with the registers
   std::vector<unsigned> worklist;
   worklist.reserve(regs.size());
-  for (std::set<unsigned>::iterator I = regs.begin(), E = regs.end();
+  for (std::map<unsigned, unsigned>::iterator I = regs.begin(), E = regs.end();
        I != E; ++I)
-    worklist.push_back(*I);
+    worklist.push_back(I->first);
   
   // Sort the registers by the DFS-in number of their defining block
   PreorderSorter PS(preorder, MRI);
@@ -269,34 +279,23 @@ StrongPHIElimination::computeDomForest(std::set<unsigned>& regs,
   return ret;
 }
 
-/// isLiveIn - helper method that determines, from a VarInfo, if a register
+/// isLiveIn - helper method that determines, from a regno, if a register
 /// is live into a block
 static bool isLiveIn(unsigned r, MachineBasicBlock* MBB,
-                     MachineRegisterInfo& MRI, LiveVariables& LV) {
-  LiveVariables::VarInfo V = LV.getVarInfo(r);
-  if (V.AliveBlocks.test(MBB->getNumber()))
-    return true;
-  
-  if (MRI.getVRegDef(r)->getParent() != MBB &&
-      V.UsedBlocks.test(MBB->getNumber()))
-    return true;
-  
-  return false;
+                     LiveIntervals& LI) {
+  LiveInterval& I = LI.getOrCreateInterval(r);
+  unsigned idx = LI.getMBBStartIdx(MBB);
+  return I.liveBeforeAndAt(idx);
 }
 
-/// isLiveOut - help method that determines, from a VarInfo, if a register is
+/// isLiveOut - help method that determines, from a regno, if a register is
 /// live out of a block.
 static bool isLiveOut(unsigned r, MachineBasicBlock* MBB,
-                      MachineRegisterInfo& MRI, LiveVariables& LV) {
-  LiveVariables::VarInfo& V = LV.getVarInfo(r);
-  if (MBB == MRI.getVRegDef(r)->getParent() ||
-      V.UsedBlocks.test(MBB->getNumber())) {
-    for (std::vector<MachineInstr*>::iterator I = V.Kills.begin(), 
-         E = V.Kills.end(); I != E; ++I)
-      if ((*I)->getParent() == MBB)
-        return false;
-    
-    return true;
+                      LiveIntervals& LI) {
+  for (MachineBasicBlock::succ_iterator PI = MBB->succ_begin(),
+       E = MBB->succ_end(); PI != E; ++PI) {
+    if (isLiveIn(r, *PI, LI))
+      return true;
   }
   
   return false;
@@ -307,7 +306,7 @@ static bool isLiveOut(unsigned r, MachineBasicBlock* MBB,
 /// registers. 0 - defined in the same block, 1 - first properly dominates
 /// second, 2 - second properly dominates first 
 static bool interferes(unsigned a, unsigned b, MachineBasicBlock* scan,
-                       LiveVariables& LV, unsigned mode) {
+                       LiveIntervals& LV, unsigned mode) {
   MachineInstr* def = 0;
   MachineInstr* kill = 0;
   
@@ -351,10 +350,10 @@ static bool interferes(unsigned a, unsigned b, MachineBasicBlock* scan,
           break;
         }
       // Store KillInsts if they match up with the definition
-      } else if (LV.KillsRegister(curr, a)) {
+      } else if (curr->killsRegister(a)) {
         if (def == MRI->getVRegDef(a)) {
           kill = curr;
-        } else if (LV.KillsRegister(curr, b)) {
+        } else if (curr->killsRegister(b)) {
           if (def == MRI->getVRegDef(b)) {
             kill = curr;
           }
@@ -373,7 +372,7 @@ static bool interferes(unsigned a, unsigned b, MachineBasicBlock* scan,
           break;
         }
       // Save KillInsts of First
-      } else if (LV.KillsRegister(curr, a)) {
+      } else if (curr->killsRegister(a)) {
         kill = curr;
       }
     // Symmetric with the above
@@ -386,7 +385,7 @@ static bool interferes(unsigned a, unsigned b, MachineBasicBlock* scan,
           interference = false;
           break;
         }
-      } else if (LV.KillsRegister(curr, b)) {
+      } else if (curr->killsRegister(b)) {
         kill = curr;
       }
     }
@@ -400,7 +399,7 @@ static bool interferes(unsigned a, unsigned b, MachineBasicBlock* scan,
 /// copies.  This method is responsible for determining which operands receive
 /// which treatment.
 void StrongPHIElimination::processBlock(MachineBasicBlock* MBB) {
-  LiveVariables& LV = getAnalysis<LiveVariables>();
+  LiveIntervals& LI = getAnalysis<LiveIntervals>();
   MachineRegisterInfo& MRI = MBB->getParent()->getRegInfo();
   
   // Holds names that have been added to a set in any PHI within this block
@@ -412,12 +411,23 @@ void StrongPHIElimination::processBlock(MachineBasicBlock* MBB) {
   while (P != MBB->end() && P->getOpcode() == TargetInstrInfo::PHI) {
     unsigned DestReg = P->getOperand(0).getReg();
 
+    // Don't both doing PHI elimination for dead PHI's.
+    if (P->registerDefIsDead(DestReg)) {
+      ++P;
+      continue;
+    }
+
+    LiveInterval& PI = LI.getOrCreateInterval(DestReg);
+    unsigned pIdx = LI.getDefIndex(LI.getInstructionIndex(P));
+    VNInfo* PVN = PI.getLiveRangeContaining(pIdx)->valno;
+    PhiValueNumber.insert(std::make_pair(DestReg, PVN->id));
+
     // PHIUnion is the set of incoming registers to the PHI node that
     // are going to be renames rather than having copies inserted.  This set
     // is refinded over the course of this function.  UnionedBlocks is the set
     // of corresponding MBBs.
-    std::set<unsigned> PHIUnion;
-    std::set<MachineBasicBlock*> UnionedBlocks;
+    std::map<unsigned, unsigned> PHIUnion;
+    SmallPtrSet<MachineBasicBlock*, 8> UnionedBlocks;
   
     // Iterate over the operands of the PHI node
     for (int i = P->getNumOperands() - 1; i >= 2; i-=2) {
@@ -435,12 +445,12 @@ void StrongPHIElimination::processBlock(MachineBasicBlock* MBB) {
       //      in this block OR
       //   5) if any two operands are defined in the same block, insert copies
       //      for one of them
-      if (isLiveIn(SrcReg, P->getParent(), MRI, LV) ||
+      if (isLiveIn(SrcReg, P->getParent(), LI) ||
           isLiveOut(P->getOperand(0).getReg(),
-                    MRI.getVRegDef(SrcReg)->getParent(), MRI, LV) ||
+                    MRI.getVRegDef(SrcReg)->getParent(), LI) ||
           ( MRI.getVRegDef(SrcReg)->getOpcode() == TargetInstrInfo::PHI &&
             isLiveIn(P->getOperand(0).getReg(),
-                     MRI.getVRegDef(SrcReg)->getParent(), MRI, LV) ) ||
+                     MRI.getVRegDef(SrcReg)->getParent(), LI) ) ||
           ProcessedNames.count(SrcReg) ||
           UnionedBlocks.count(MRI.getVRegDef(SrcReg)->getParent())) {
         
@@ -450,7 +460,13 @@ void StrongPHIElimination::processBlock(MachineBasicBlock* MBB) {
         UsedByAnother.insert(SrcReg);
       } else {
         // Otherwise, add it to the renaming set
-        PHIUnion.insert(SrcReg);
+        LiveInterval& I = LI.getOrCreateInterval(SrcReg);
+        unsigned idx = LI.getMBBEndIdx(P->getOperand(i).getMBB());
+        VNInfo* VN = I.getLiveRangeContaining(idx)->valno;
+        
+        assert(VN && "No VNInfo for register?");
+        
+        PHIUnion.insert(std::make_pair(SrcReg, VN->id));
         UnionedBlocks.insert(MRI.getVRegDef(SrcReg)->getParent());
       }
     }
@@ -467,8 +483,17 @@ void StrongPHIElimination::processBlock(MachineBasicBlock* MBB) {
     std::vector<std::pair<unsigned, unsigned> > localInterferences;
     processPHIUnion(P, PHIUnion, DF, localInterferences);
     
+    // If one of the inputs is defined in the same block as the current PHI
+    // then we need to check for a local interference between that input and
+    // the PHI.
+    for (std::map<unsigned, unsigned>::iterator I = PHIUnion.begin(),
+         E = PHIUnion.end(); I != E; ++I)
+      if (MRI.getVRegDef(I->first)->getParent() == P->getParent())
+        localInterferences.push_back(std::make_pair(I->first,
+                                                    P->getOperand(0).getReg()));
+    
     // The dominator forest walk may have returned some register pairs whose
-    // interference cannot be determines from dominator analysis.  We now 
+    // interference cannot be determined from dominator analysis.  We now 
     // examine these pairs for local interferences.
     for (std::vector<std::pair<unsigned, unsigned> >::iterator I =
         localInterferences.begin(), E = localInterferences.end(); I != E; ++I) {
@@ -494,7 +519,7 @@ void StrongPHIElimination::processBlock(MachineBasicBlock* MBB) {
       }
       
       // If there's an interference, we need to insert  copies
-      if (interferes(p.first, p.second, scan, LV, mode)) {
+      if (interferes(p.first, p.second, scan, LI, mode)) {
         // Insert copies for First
         for (int i = P->getNumOperands() - 1; i >= 2; i-=2) {
           if (P->getOperand(i-1).getReg() == p.first) {
@@ -511,23 +536,25 @@ void StrongPHIElimination::processBlock(MachineBasicBlock* MBB) {
       }
     }
     
-    // Add the renaming set for this PHI node to our overal renaming information
+    // Add the renaming set for this PHI node to our overall renaming information
     RenameSets.insert(std::make_pair(P->getOperand(0).getReg(), PHIUnion));
     
     // Remember which registers are already renamed, so that we don't try to 
     // rename them for another PHI node in this block
-    ProcessedNames.insert(PHIUnion.begin(), PHIUnion.end());
+    for (std::map<unsigned, unsigned>::iterator I = PHIUnion.begin(),
+         E = PHIUnion.end(); I != E; ++I)
+      ProcessedNames.insert(I->first);
     
     ++P;
   }
 }
 
-/// processPHIUnion - Take a set of candidate registers to be coallesced when
+/// processPHIUnion - Take a set of candidate registers to be coalesced when
 /// decomposing the PHI instruction.  Use the DominanceForest to remove the ones
 /// that are known to interfere, and flag others that need to be checked for
 /// local interferences.
 void StrongPHIElimination::processPHIUnion(MachineInstr* Inst,
-                                           std::set<unsigned>& PHIUnion,
+                                        std::map<unsigned, unsigned>& PHIUnion,
                         std::vector<StrongPHIElimination::DomForestNode*>& DF,
                         std::vector<std::pair<unsigned, unsigned> >& locals) {
   
@@ -537,7 +564,7 @@ void StrongPHIElimination::processPHIUnion(MachineInstr* Inst,
   // Code is still in SSA form, so we can use MRI::getVRegDef()
   MachineRegisterInfo& MRI = Inst->getParent()->getParent()->getRegInfo();
   
-  LiveVariables& LV = getAnalysis<LiveVariables>();
+  LiveIntervals& LI = getAnalysis<LiveIntervals>();
   unsigned DestReg = Inst->getOperand(0).getReg();
   
   // DF walk on the DomForest
@@ -550,9 +577,14 @@ void StrongPHIElimination::processPHIUnion(MachineInstr* Inst,
     for (DomForestNode::iterator CI = DFNode->begin(), CE = DFNode->end();
          CI != CE; ++CI) {
       DomForestNode* child = *CI;   
-        
+      
+      // If the current node is live-out of the defining block of one of its
+      // children, insert a copy for it.  NOTE: The paper actually calls for
+      // a more elaborate heuristic for determining whether to insert copies
+      // for the child or the parent.  In the interest of simplicity, we're
+      // just always choosing the parent.
       if (isLiveOut(DFNode->getReg(),
-          MRI.getVRegDef(child->getReg())->getParent(), MRI, LV)) {
+          MRI.getVRegDef(child->getReg())->getParent(), LI)) {
         // Insert copies for parent
         for (int i = Inst->getNumOperands() - 1; i >= 2; i-=2) {
           if (Inst->getOperand(i-1).getReg() == DFNode->getReg()) {
@@ -565,9 +597,11 @@ void StrongPHIElimination::processPHIUnion(MachineInstr* Inst,
             PHIUnion.erase(SrcReg);
           }
         }
+      
+      // If a node is live-in to the defining block of one of its children, but
+      // not live-out, then we need to scan that block for local interferences.
       } else if (isLiveIn(DFNode->getReg(),
-                          MRI.getVRegDef(child->getReg())->getParent(),
-                          MRI, LV) ||
+                          MRI.getVRegDef(child->getReg())->getParent(), LI) ||
                  MRI.getVRegDef(DFNode->getReg())->getParent() ==
                                  MRI.getVRegDef(child->getReg())->getParent()) {
         // Add (p, c) to possible local interferences
@@ -604,7 +638,7 @@ void StrongPHIElimination::ScheduleCopies(MachineBasicBlock* MBB,
     map.insert(std::make_pair(I->first, I->first));
     map.insert(std::make_pair(I->second, I->second));
          
-    if (!UsedByAnother.count(I->first)) {
+    if (!UsedByAnother.count(I->second)) {
       worklist.insert(*I);
       
       // Avoid iterator invalidation
@@ -616,7 +650,7 @@ void StrongPHIElimination::ScheduleCopies(MachineBasicBlock* MBB,
     }
   }
   
-  LiveVariables& LV = getAnalysis<LiveVariables>();
+  LiveIntervals& LI = getAnalysis<LiveIntervals>();
   MachineFunction* MF = MBB->getParent();
   MachineRegisterInfo& MRI = MF->getRegInfo();
   const TargetInstrInfo *TII = MF->getTarget().getInstrInfo();
@@ -629,7 +663,7 @@ void StrongPHIElimination::ScheduleCopies(MachineBasicBlock* MBB,
       
       const TargetRegisterClass *RC = MF->getRegInfo().getRegClass(curr.first);
       
-      if (isLiveOut(curr.second, MBB, MRI, LV)) {
+      if (isLiveOut(curr.second, MBB, LI)) {
         // Create a temporary
         unsigned t = MF->getRegInfo().createVirtualRegister(RC);
         
@@ -687,7 +721,7 @@ void StrongPHIElimination::ScheduleCopies(MachineBasicBlock* MBB,
 
 /// InsertCopies - insert copies into MBB and all of its successors
 void StrongPHIElimination::InsertCopies(MachineBasicBlock* MBB,
-                                        std::set<MachineBasicBlock*>& visited) {
+                                 SmallPtrSet<MachineBasicBlock*, 16>& visited) {
   visited.insert(MBB);
   
   std::set<unsigned> pushed;
@@ -717,7 +751,111 @@ void StrongPHIElimination::InsertCopies(MachineBasicBlock* MBB,
     Stacks[*I].pop_back();
 }
 
+/// ComputeUltimateVN - Assuming we are going to join two live intervals,
+/// compute what the resultant value numbers for each value in the input two
+/// ranges will be.  This is complicated by copies between the two which can
+/// and will commonly cause multiple value numbers to be merged into one.
+///
+/// VN is the value number that we're trying to resolve.  InstDefiningValue
+/// keeps track of the new InstDefiningValue assignment for the result
+/// LiveInterval.  ThisFromOther/OtherFromThis are sets that keep track of
+/// whether a value in this or other is a copy from the opposite set.
+/// ThisValNoAssignments/OtherValNoAssignments keep track of value #'s that have
+/// already been assigned.
+///
+/// ThisFromOther[x] - If x is defined as a copy from the other interval, this
+/// contains the value number the copy is from.
+///
+static unsigned ComputeUltimateVN(VNInfo *VNI,
+                                  SmallVector<VNInfo*, 16> &NewVNInfo,
+                                  DenseMap<VNInfo*, VNInfo*> &ThisFromOther,
+                                  DenseMap<VNInfo*, VNInfo*> &OtherFromThis,
+                                  SmallVector<int, 16> &ThisValNoAssignments,
+                                  SmallVector<int, 16> &OtherValNoAssignments) {
+  unsigned VN = VNI->id;
+
+  // If the VN has already been computed, just return it.
+  if (ThisValNoAssignments[VN] >= 0)
+    return ThisValNoAssignments[VN];
+//  assert(ThisValNoAssignments[VN] != -2 && "Cyclic case?");
+
+  // If this val is not a copy from the other val, then it must be a new value
+  // number in the destination.
+  DenseMap<VNInfo*, VNInfo*>::iterator I = ThisFromOther.find(VNI);
+  if (I == ThisFromOther.end()) {
+    NewVNInfo.push_back(VNI);
+    return ThisValNoAssignments[VN] = NewVNInfo.size()-1;
+  }
+  VNInfo *OtherValNo = I->second;
+
+  // Otherwise, this *is* a copy from the RHS.  If the other side has already
+  // been computed, return it.
+  if (OtherValNoAssignments[OtherValNo->id] >= 0)
+    return ThisValNoAssignments[VN] = OtherValNoAssignments[OtherValNo->id];
+  
+  // Mark this value number as currently being computed, then ask what the
+  // ultimate value # of the other value is.
+  ThisValNoAssignments[VN] = -2;
+  unsigned UltimateVN =
+    ComputeUltimateVN(OtherValNo, NewVNInfo, OtherFromThis, ThisFromOther,
+                      OtherValNoAssignments, ThisValNoAssignments);
+  return ThisValNoAssignments[VN] = UltimateVN;
+}
+
+void StrongPHIElimination::mergeLiveIntervals(unsigned primary,
+                                              unsigned secondary,
+                                              unsigned secondaryVN) {
+  
+  LiveIntervals& LI = getAnalysis<LiveIntervals>();
+  LiveInterval& LHS = LI.getOrCreateInterval(primary);
+  LiveInterval& RHS = LI.getOrCreateInterval(secondary);
+  
+  // Compute the final value assignment, assuming that the live ranges can be
+  // coalesced.
+  SmallVector<int, 16> LHSValNoAssignments;
+  SmallVector<int, 16> RHSValNoAssignments;
+  SmallVector<VNInfo*, 16> NewVNInfo;
+  
+  LHSValNoAssignments.resize(LHS.getNumValNums(), -1);
+  RHSValNoAssignments.resize(RHS.getNumValNums(), -1);
+  NewVNInfo.reserve(LHS.getNumValNums() + RHS.getNumValNums());
+  
+  for (LiveInterval::vni_iterator I = LHS.vni_begin(), E = LHS.vni_end();
+       I != E; ++I) {
+    VNInfo *VNI = *I;
+    unsigned VN = VNI->id;
+    if (LHSValNoAssignments[VN] >= 0 || VNI->def == ~1U) 
+      continue;
+    
+    NewVNInfo.push_back(VNI);
+    LHSValNoAssignments[VN] = NewVNInfo.size()-1;
+  }
+  
+  for (LiveInterval::vni_iterator I = RHS.vni_begin(), E = RHS.vni_end();
+       I != E; ++I) {
+    VNInfo *VNI = *I;
+    unsigned VN = VNI->id;
+    if (RHSValNoAssignments[VN] >= 0 || VNI->def == ~1U)
+      continue;
+      
+    NewVNInfo.push_back(VNI);
+    RHSValNoAssignments[VN] = NewVNInfo.size()-1;
+  }
+
+  // If we get here, we know that we can coalesce the live ranges.  Ask the
+  // intervals to coalesce themselves now.
+
+  LHS.join(RHS, &LHSValNoAssignments[0], &RHSValNoAssignments[0], NewVNInfo);
+  LI.removeInterval(secondary);
+  
+  // The valno that was previously the input to the PHI node
+  // now has a PHIKill.
+  LHS.getValNumInfo(RHSValNoAssignments[secondaryVN])->hasPHIKill = true;
+}
+
 bool StrongPHIElimination::runOnMachineFunction(MachineFunction &Fn) {
+  LiveIntervals& LI = getAnalysis<LiveIntervals>();
+  
   // Compute DFS numbers of each block
   computeDFS(Fn);
   
@@ -729,25 +867,54 @@ bool StrongPHIElimination::runOnMachineFunction(MachineFunction &Fn) {
   
   // Insert copies
   // FIXME: This process should probably preserve LiveVariables
-  std::set<MachineBasicBlock*> visited;
+  SmallPtrSet<MachineBasicBlock*, 16> visited;
   InsertCopies(Fn.begin(), visited);
   
   // Perform renaming
-  typedef std::map<unsigned, std::set<unsigned> > RenameSetType;
+  typedef std::map<unsigned, std::map<unsigned, unsigned> > RenameSetType;
   for (RenameSetType::iterator I = RenameSets.begin(), E = RenameSets.end();
        I != E; ++I)
-    for (std::set<unsigned>::iterator SI = I->second.begin(),
-         SE = I->second.end(); SI != SE; ++SI)
-      Fn.getRegInfo().replaceRegWith(*SI, I->first);
+    for (std::map<unsigned, unsigned>::iterator SI = I->second.begin(),
+         SE = I->second.end(); SI != SE; ++SI) {
+      mergeLiveIntervals(I->first, SI->first, SI->second);
+      Fn.getRegInfo().replaceRegWith(SI->first, I->first);
+    }
   
   // FIXME: Insert last-minute copies
   
   // Remove PHIs
-  for (MachineFunction::iterator I = Fn.begin(), E = Fn.end(); I != E; ++I)
+  std::vector<MachineInstr*> phis;
+  for (MachineFunction::iterator I = Fn.begin(), E = Fn.end(); I != E; ++I) {
     for (MachineBasicBlock::iterator BI = I->begin(), BE = I->end();
          BI != BE; ++BI)
       if (BI->getOpcode() == TargetInstrInfo::PHI)
-        BI->eraseFromParent();
+        phis.push_back(BI);
+  }
   
-  return false;
+  for (std::vector<MachineInstr*>::iterator I = phis.begin(), E = phis.end();
+       I != E; ) {
+    MachineInstr* PInstr = *(I++);
+    
+    // If this is a dead PHI node, then remove it from LiveIntervals.
+    unsigned DestReg = PInstr->getOperand(0).getReg();
+    LiveInterval& PI = LI.getInterval(DestReg);
+    if (PInstr->registerDefIsDead(DestReg)) {
+      if (PI.containsOneValue()) {
+        LI.removeInterval(DestReg);
+      } else {
+        unsigned idx = LI.getDefIndex(LI.getInstructionIndex(PInstr));
+        PI.removeRange(*PI.getLiveRangeContaining(idx), true);
+      }
+    } else {
+      // If the PHI is not dead, then the valno defined by the PHI
+      // now has an unknown def.
+      unsigned idx = LI.getDefIndex(LI.getInstructionIndex(PInstr));
+      PI.getLiveRangeContaining(idx)->valno->def = ~0U;
+    }
+    
+    LI.RemoveMachineInstrFromMaps(PInstr);
+    PInstr->eraseFromParent();
+  }
+  
+  return true;
 }
