@@ -316,7 +316,7 @@ static Value *RemapOperand(const Value *In,
   
   // Cache the mapping in our local map structure
   if (Result) {
-    ValueMap.insert(std::make_pair(In, Result));
+    ValueMap[In] = Result;
     return Result;
   }
   
@@ -361,6 +361,9 @@ static void CopyGVAttributes(GlobalValue *DestGV, const GlobalValue *SrcGV) {
     DestF->setParamAttrs(SrcF->getParamAttrs());
     if (SrcF->hasCollector())
       DestF->setCollector(SrcF->getCollector());
+  } else if (const GlobalVariable *SrcVar = dyn_cast<GlobalVariable>(SrcGV)) {
+    GlobalVariable *DestVar = cast<GlobalVariable>(DestGV);
+    DestVar->setThreadLocal(SrcVar->isThreadLocal());
   }
 }
 
@@ -371,7 +374,7 @@ static void CopyGVAttributes(GlobalValue *DestGV, const GlobalValue *SrcGV) {
 /// one), and computes whether this linkage is an error or not. It also performs
 /// visibility checks: we cannot link together two symbols with different
 /// visibilities.
-static bool GetLinkageResult(GlobalValue *Dest, GlobalValue *Src,
+static bool GetLinkageResult(GlobalValue *Dest, const GlobalValue *Src,
                              GlobalValue::LinkageTypes &LT, bool &LinkFromSrc,
                              std::string *Err) {
   assert((!Dest || !Src->hasInternalLinkage()) &&
@@ -381,7 +384,7 @@ static bool GetLinkageResult(GlobalValue *Dest, GlobalValue *Src,
     LinkFromSrc = true;
     LT = Src->getLinkage();
   } else if (Src->isDeclaration()) {
-    // If Src is external or if both Src & Drc are external..  Just link the
+    // If Src is external or if both Src & Dest are external..  Just link the
     // external globals, we aren't adding anything.
     if (Src->hasDLLImportLinkage()) {
       // If one of GVs has DLLImport linkage, result should be dllimport'ed.
@@ -451,16 +454,17 @@ static bool GetLinkageResult(GlobalValue *Dest, GlobalValue *Src,
 
 // LinkGlobals - Loop through the global variables in the src module and merge
 // them into the dest module.
-static bool LinkGlobals(Module *Dest, Module *Src,
+static bool LinkGlobals(Module *Dest, const Module *Src,
                         std::map<const Value*, Value*> &ValueMap,
                     std::multimap<std::string, GlobalVariable *> &AppendingVars,
                         std::string *Err) {
   // Loop over all of the globals in the src module, mapping them over as we go
-  for (Module::global_iterator I = Src->global_begin(), E = Src->global_end();
+  for (Module::const_global_iterator I = Src->global_begin(), E = Src->global_end();
        I != E; ++I) {
-    GlobalVariable *SGV = I;
-    GlobalVariable *DGV = 0;
-    // Check to see if may have to link the global.
+    const GlobalVariable *SGV = I;
+    GlobalValue *DGV = 0;
+
+    // Check to see if may have to link the global with the global
     if (SGV->hasName() && !SGV->hasInternalLinkage()) {
       DGV = Dest->getGlobalVariable(SGV->getName());
       if (DGV && DGV->getType() != SGV->getType())
@@ -469,6 +473,15 @@ static bool LinkGlobals(Module *Dest, Module *Src,
                               &Dest->getTypeSymbolTable(), "");
     }
 
+    // Check to see if may have to link the global with the alias
+    if (!DGV && SGV->hasName() && !SGV->hasInternalLinkage()) {
+      DGV = Dest->getNamedAlias(SGV->getName());
+      if (DGV && DGV->getType() != SGV->getType())
+        // If types don't agree due to opaque types, try to resolve them.
+        RecursiveResolveTypes(SGV->getType(), DGV->getType(), 
+                              &Dest->getTypeSymbolTable(), "");
+    }
+
     if (DGV && DGV->hasInternalLinkage())
       DGV = 0;
 
@@ -488,7 +501,7 @@ static bool LinkGlobals(Module *Dest, Module *Src,
       GlobalVariable *NewDGV =
         new GlobalVariable(SGV->getType()->getElementType(),
                            SGV->isConstant(), SGV->getLinkage(), /*init*/0,
-                           SGV->getName(), Dest, SGV->isThreadLocal());
+                           SGV->getName(), Dest);
       // Propagate alignment, visibility and section info.
       CopyGVAttributes(NewDGV, SGV);
 
@@ -499,7 +512,8 @@ static bool LinkGlobals(Module *Dest, Module *Src,
         ForceRenaming(NewDGV, SGV->getName());
 
       // Make sure to remember this mapping...
-      ValueMap.insert(std::make_pair(SGV, NewDGV));
+      ValueMap[SGV] = NewDGV;
+
       if (SGV->hasAppendingLinkage())
         // Keep track that this is an appending variable...
         AppendingVars.insert(std::make_pair(SGV->getName(), NewDGV));
@@ -511,81 +525,219 @@ static bool LinkGlobals(Module *Dest, Module *Src,
       GlobalVariable *NewDGV =
         new GlobalVariable(SGV->getType()->getElementType(),
                            SGV->isConstant(), SGV->getLinkage(), /*init*/0,
-                           "", Dest, SGV->isThreadLocal());
+                           "", Dest);
 
-      // Propagate alignment, section and visibility  info.
+      // Set alignment allowing CopyGVAttributes merge it with alignment of SGV.
       NewDGV->setAlignment(DGV->getAlignment());
+      // Propagate alignment, section and visibility info.
       CopyGVAttributes(NewDGV, SGV);
 
       // Make sure to remember this mapping...
-      ValueMap.insert(std::make_pair(SGV, NewDGV));
+      ValueMap[SGV] = NewDGV;
 
       // Keep track that this is an appending variable...
       AppendingVars.insert(std::make_pair(SGV->getName(), NewDGV));
-    } else {
-      // Propagate alignment, section, and visibility info.
-      CopyGVAttributes(DGV, SGV);
-
-      // Otherwise, perform the mapping as instructed by GetLinkageResult.  If
-      // the types don't match, and if we are to link from the source, nuke DGV
-      // and create a new one of the appropriate type.
-      if (SGV->getType() != DGV->getType() && LinkFromSrc) {
-        GlobalVariable *NewDGV =
-          new GlobalVariable(SGV->getType()->getElementType(),
-                             DGV->isConstant(), DGV->getLinkage());
-        NewDGV->setThreadLocal(DGV->isThreadLocal());
-        CopyGVAttributes(NewDGV, DGV);
-        Dest->getGlobalList().insert(DGV, NewDGV);
-        DGV->replaceAllUsesWith(
-            ConstantExpr::getBitCast(NewDGV, DGV->getType()));
-        DGV->eraseFromParent();
-        NewDGV->setName(SGV->getName());
-        DGV = NewDGV;
-      }
-
-      DGV->setLinkage(NewLinkage);
-
+    } else if (GlobalAlias *DGA = dyn_cast<GlobalAlias>(DGV)) {
+      // SGV is global, but DGV is alias. The only valid mapping is when SGV is
+      // external declaration, which is effectively a no-op. Also make sure
+      // linkage calculation was correct.
+      if (SGV->isDeclaration() && !LinkFromSrc) {
+        // Make sure to remember this mapping...
+        ValueMap[SGV] = DGA;
+      } else
+        return Error(Err, "Global-Alias Collision on '" + SGV->getName() +
+                     "': symbol multiple defined");
+    } else if (GlobalVariable *DGVar = dyn_cast<GlobalVariable>(DGV)) {
+      // Otherwise, perform the global-global mapping as instructed by
+      // GetLinkageResult.
       if (LinkFromSrc) {
-        // Inherit const as appropriate
-        DGV->setConstant(SGV->isConstant());
-        DGV->setInitializer(0);
-      } else {
-        if (SGV->isConstant() && !DGV->isConstant()) {
-          if (DGV->isDeclaration())
-            DGV->setConstant(true);
+        // Propagate alignment, section, and visibility info.
+        CopyGVAttributes(DGVar, SGV);
+
+        // If the types don't match, and if we are to link from the source, nuke
+        // DGV and create a new one of the appropriate type.
+        if (SGV->getType() != DGVar->getType()) {
+          GlobalVariable *NewDGV =
+            new GlobalVariable(SGV->getType()->getElementType(),
+                               DGVar->isConstant(), DGVar->getLinkage(),
+                               /*init*/0, DGVar->getName(), Dest);
+          CopyGVAttributes(NewDGV, DGVar);
+          DGV->replaceAllUsesWith(ConstantExpr::getBitCast(NewDGV,
+                                                           DGVar->getType()));
+          // DGVar will conflict with NewDGV because they both had the same
+          // name. We must erase this now so ForceRenaming doesn't assert
+          // because DGV might not have internal linkage.
+          DGVar->eraseFromParent();
+
+          // If the symbol table renamed the global, but it is an externally
+          // visible symbol, DGV must be an existing global with internal
+          // linkage. Rename it.
+          if (NewDGV->getName() != SGV->getName() &&
+              !NewDGV->hasInternalLinkage())
+            ForceRenaming(NewDGV, SGV->getName());
+
+          DGVar = NewDGV;
         }
-        SGV->setLinkage(GlobalValue::ExternalLinkage);
-        SGV->setInitializer(0);
+
+        // Inherit const as appropriate
+        DGVar->setConstant(SGV->isConstant());
+
+        // Set initializer to zero, so we can link the stuff later
+        DGVar->setInitializer(0);
+      } else {
+        // Special case for const propagation
+        if (DGVar->isDeclaration() && SGV->isConstant() && !DGVar->isConstant())
+          DGVar->setConstant(true);
       }
 
-      ValueMap.insert(
-        std::make_pair(SGV, ConstantExpr::getBitCast(DGV, SGV->getType())));
+      // Set calculated linkage
+      DGVar->setLinkage(NewLinkage);
+
+      // Make sure to remember this mapping...
+      ValueMap[SGV] = ConstantExpr::getBitCast(DGVar, SGV->getType());
     }
   }
   return false;
 }
 
+static GlobalValue::LinkageTypes
+CalculateAliasLinkage(const GlobalValue *SGV, const GlobalValue *DGV) {
+  if (SGV->hasExternalLinkage() || DGV->hasExternalLinkage())
+    return GlobalValue::ExternalLinkage;
+  else if (SGV->hasWeakLinkage() || DGV->hasWeakLinkage())
+    return GlobalValue::WeakLinkage;
+  else {
+    assert(SGV->hasInternalLinkage() && DGV->hasInternalLinkage() &&
+           "Unexpected linkage type");
+    return GlobalValue::InternalLinkage;
+  }
+}
+
 // LinkAlias - Loop through the alias in the src module and link them into the
-// dest module.
-static bool LinkAlias(Module *Dest, const Module *Src, std::string *Err) {
+// dest module. We're assuming, that all functions/global variables were already
+// linked in.
+static bool LinkAlias(Module *Dest, const Module *Src,
+                      std::map<const Value*, Value*> &ValueMap,
+                      std::string *Err) {
   // Loop over all alias in the src module
   for (Module::const_alias_iterator I = Src->alias_begin(),
          E = Src->alias_end(); I != E; ++I) {
-    const GlobalAlias *GA = I;
+    const GlobalAlias *SGA = I;
+    const GlobalValue *SAliasee = SGA->getAliasedGlobal();
+    GlobalAlias *NewGA = NULL;
 
-    GlobalValue *NewAliased = NULL;
-    const GlobalValue *Aliased = GA->getAliasedGlobal();
-    if (isa<GlobalVariable>(*Aliased))
-      NewAliased = Dest->getGlobalVariable(Aliased->getName());
-    else if (isa<Function>(*Aliased))
-      NewAliased = Dest->getFunction(Aliased->getName());
-    // FIXME: we should handle the bitcast alias.
-    assert(NewAliased && "Can't find the aliased GV.");
+    // Globals were already linked, thus we can just query ValueMap for variant
+    // of SAliasee in Dest
+    std::map<const Value*,Value*>::const_iterator VMI = ValueMap.find(SAliasee);
+    assert(VMI != ValueMap.end() && "Aliasee not linked");
+    GlobalValue* DAliasee = cast<GlobalValue>(VMI->second);
 
-    GlobalAlias *NewGA = new GlobalAlias(GA->getType(), GA->getLinkage(),
-                                         GA->getName(), NewAliased, Dest);
-    CopyGVAttributes(NewGA, GA);
+    // Try to find something 'similar' to SGA in destination module.
+    if (GlobalAlias *DGA = Dest->getNamedAlias(SGA->getName())) {
+      // If types don't agree due to opaque types, try to resolve them.
+      if (RecursiveResolveTypes(SGA->getType(), DGA->getType(),
+                                &Dest->getTypeSymbolTable(), ""))
+        return Error(Err, "Alias Collision on '" + SGA->getName()+
+                     "': aliases have different types");
+
+      // Now types are known to be the same, check whether aliasees equal. As
+      // globals are already linked we just need query ValueMap to find the
+      // mapping.
+      if (DAliasee == DGA->getAliasedGlobal()) {
+        // This is just two copies of the same alias. Propagate linkage, if
+        // necessary.
+        DGA->setLinkage(CalculateAliasLinkage(SGA, DGA));
+
+        NewGA = DGA;
+        // Proceed to 'common' steps
+      } else
+        return Error(Err, "Alias Collision on '"  + SGA->getName()+
+                     "': aliases have different aliasees");
+    } else if (GlobalVariable *DGV = Dest->getGlobalVariable(SGA->getName())) {
+      RecursiveResolveTypes(SGA->getType(), DGV->getType(),
+                            &Dest->getTypeSymbolTable(), "");
+
+      // The only allowed way is to link alias with external declaration.
+      if (DGV->isDeclaration()) {
+        // But only if aliasee is global too...
+        if (!isa<GlobalVariable>(DAliasee))
+            return Error(Err, "Global-Alias Collision on '" + SGA->getName() +
+                         "': aliasee is not global variable");
+
+        NewGA = new GlobalAlias(SGA->getType(), SGA->getLinkage(),
+                                SGA->getName(), DAliasee, Dest);
+        CopyGVAttributes(NewGA, SGA);
+
+        // Any uses of DGV need to change to NewGA, with cast, if needed.
+        if (SGA->getType() != DGV->getType())
+          DGV->replaceAllUsesWith(ConstantExpr::getBitCast(NewGA,
+                                                           DGV->getType()));
+        else
+          DGV->replaceAllUsesWith(NewGA);
+
+        // DGV will conflict with NewGA because they both had the same
+        // name. We must erase this now so ForceRenaming doesn't assert
+        // because DGV might not have internal linkage.
+        DGV->eraseFromParent();
+
+        // Proceed to 'common' steps
+      } else
+        return Error(Err, "Global-Alias Collision on '" + SGA->getName() +
+                     "': symbol multiple defined");
+    } else if (Function *DF = Dest->getFunction(SGA->getName())) {
+      RecursiveResolveTypes(SGA->getType(), DF->getType(),
+                            &Dest->getTypeSymbolTable(), "");
+
+      // The only allowed way is to link alias with external declaration.
+      if (DF->isDeclaration()) {
+        // But only if aliasee is function too...
+        if (!isa<Function>(DAliasee))
+            return Error(Err, "Function-Alias Collision on '" + SGA->getName() +
+                         "': aliasee is not function");
+
+        NewGA = new GlobalAlias(SGA->getType(), SGA->getLinkage(),
+                                SGA->getName(), DAliasee, Dest);
+        CopyGVAttributes(NewGA, SGA);
+
+        // Any uses of DF need to change to NewGA, with cast, if needed.
+        if (SGA->getType() != DF->getType())
+          DF->replaceAllUsesWith(ConstantExpr::getBitCast(NewGA,
+                                                          DF->getType()));
+        else
+          DF->replaceAllUsesWith(NewGA);
+
+        // DF will conflict with NewGA because they both had the same
+        // name. We must erase this now so ForceRenaming doesn't assert
+        // because DF might not have internal linkage.
+        DF->eraseFromParent();
+
+        // Proceed to 'common' steps
+      } else
+        return Error(Err, "Function-Alias Collision on '" + SGA->getName() +
+                     "': symbol multiple defined");
+    } else {
+      // Nothing similar found, just copy alias into destination module.
+
+      NewGA = new GlobalAlias(SGA->getType(), SGA->getLinkage(),
+                              SGA->getName(), DAliasee, Dest);
+      CopyGVAttributes(NewGA, SGA);
+
+      // Proceed to 'common' steps
+    }
+
+    assert(NewGA && "No alias was created in destination module!");
+
+    // If the symbol table renamed the alias, but it is an externally visible
+    // symbol, DGV must be an global value with internal linkage. Rename it.
+    if (NewGA->getName() != SGA->getName() &&
+        !NewGA->hasInternalLinkage())
+      ForceRenaming(NewGA, SGA->getName());
+
+    // Remember this mapping so uses in the source module get remapped
+    // later by RemapOperand.
+    ValueMap[SGA] = NewGA;
   }
+
   return false;
 }
 
@@ -606,13 +758,13 @@ static bool LinkGlobalInits(Module *Dest, const Module *Src,
       Constant *SInit =
         cast<Constant>(RemapOperand(SGV->getInitializer(), ValueMap));
 
-      GlobalVariable *DGV = cast<GlobalVariable>(ValueMap[SGV]);
+      GlobalVariable *DGV =
+        cast<GlobalVariable>(ValueMap[SGV]->stripPointerCasts());
       if (DGV->hasInitializer()) {
         if (SGV->hasExternalLinkage()) {
           if (DGV->getInitializer() != SInit)
-            return Error(Err, "Global Variable Collision on '" +
-                         ToStr(SGV->getType(), Src) +"':%"+SGV->getName()+
-                         " - Global variables have different initializers");
+            return Error(Err, "Global Variable Collision on '" + SGV->getName() +
+                         "': global variables have different initializers");
         } else if (DGV->hasLinkOnceLinkage() || DGV->hasWeakLinkage()) {
           // Nothing is required, mapped values will take the new global
           // automatically.
@@ -671,8 +823,8 @@ static bool LinkFunctionProtos(Module *Dest, const Module *Src,
         // We have a definition of the same name but different type in the
         // source module. Copy the prototype to the destination and replace
         // uses of the destination's prototype with the new prototype.
-        Function *NewDF = new Function(SF->getFunctionType(), SF->getLinkage(),
-                                       SF->getName(), Dest);
+        Function *NewDF = Function::Create(SF->getFunctionType(), SF->getLinkage(),
+                                           SF->getName(), Dest);
         CopyGVAttributes(NewDF, SF);
 
         // Any uses of DF need to change to NewDF, with cast
@@ -707,8 +859,8 @@ static bool LinkFunctionProtos(Module *Dest, const Module *Src,
     } else if (!DF || SF->hasInternalLinkage() || DF->hasInternalLinkage()) {
       // Function does not already exist, simply insert an function signature
       // identical to SF into the dest module.
-      Function *NewDF = new Function(SF->getFunctionType(), SF->getLinkage(),
-                                     SF->getName(), Dest);
+      Function *NewDF = Function::Create(SF->getFunctionType(), SF->getLinkage(),
+                                         SF->getName(), Dest);
       CopyGVAttributes(NewDF, SF);
 
       // If the LLVM runtime renamed the function, but it is an externally
@@ -718,7 +870,7 @@ static bool LinkFunctionProtos(Module *Dest, const Module *Src,
         ForceRenaming(NewDF, SF->getName());
 
       // ... and remember this mapping...
-      ValueMap.insert(std::make_pair(SF, NewDF));
+      ValueMap[SF] = NewDF;
     } else if (SF->isDeclaration()) {
       // If SF is a declaration or if both SF & DF are declarations, just link 
       // the declarations, we aren't adding anything.
@@ -728,7 +880,7 @@ static bool LinkFunctionProtos(Module *Dest, const Module *Src,
           DF->setLinkage(SF->getLinkage());          
         }        
       } else {
-        ValueMap.insert(std::make_pair(SF, DF));
+        ValueMap[SF] = DF;
       }      
     } else if (DF->isDeclaration() && !DF->hasDLLImportLinkage()) {
       // If DF is external but SF is not...
@@ -739,7 +891,7 @@ static bool LinkFunctionProtos(Module *Dest, const Module *Src,
       DF->setVisibility(SF->getVisibility());
     } else if (SF->hasWeakLinkage() || SF->hasLinkOnceLinkage()) {
       // At this point we know that DF has LinkOnce, Weak, or External* linkage.
-      ValueMap.insert(std::make_pair(SF, DF));
+      ValueMap[SF] = DF;
 
       // Linkonce+Weak = Weak
       // *+External Weak = *
@@ -748,7 +900,7 @@ static bool LinkFunctionProtos(Module *Dest, const Module *Src,
         DF->setLinkage(SF->getLinkage());
     } else if (DF->hasWeakLinkage() || DF->hasLinkOnceLinkage()) {
       // At this point we know that SF has LinkOnce or External* linkage.
-      ValueMap.insert(std::make_pair(SF, DF));
+      ValueMap[SF] = DF;
       if (!SF->hasLinkOnceLinkage() && !SF->hasExternalWeakLinkage())
         // Don't inherit linkonce & external weak linkage
         DF->setLinkage(SF->getLinkage());
@@ -782,7 +934,7 @@ static bool LinkFunctionBody(Function *Dest, Function *Src,
     DI->setName(I->getName());  // Copy the name information over...
 
     // Add a mapping to our local map
-    ValueMap.insert(std::make_pair(I, DI));
+    ValueMap[I] = DI;
   }
 
   // Splice the body of the source function into the dest function.
@@ -934,6 +1086,15 @@ static bool LinkAppendingVars(Module *M,
   return false;
 }
 
+static bool ResolveAliases(Module *Dest) {
+  for (Module::alias_iterator I = Dest->alias_begin(), E = Dest->alias_end();
+       I != E; ++I)
+    if (const GlobalValue *GV = I->resolveAliasedGlobal())
+      if (!GV->isDeclaration())
+        I->replaceAllUsesWith(const_cast<GlobalValue*>(GV));
+
+  return false;
+}
 
 // LinkModules - This function links two modules together, with the resulting
 // left module modified to be the composite of the two input modules.  If an
@@ -951,21 +1112,24 @@ Linker::LinkModules(Module *Dest, Module *Src, std::string *ErrorMsg) {
     } else {
       std::string DataLayout;
 
-      if (Dest->getEndianness() == Module::AnyEndianness)
+      if (Dest->getEndianness() == Module::AnyEndianness) {
         if (Src->getEndianness() == Module::BigEndian)
           DataLayout.append("E");
         else if (Src->getEndianness() == Module::LittleEndian)
           DataLayout.append("e");
-      if (Dest->getPointerSize() == Module::AnyPointerSize)
+      }
+
+      if (Dest->getPointerSize() == Module::AnyPointerSize) {
         if (Src->getPointerSize() == Module::Pointer64)
           DataLayout.append(DataLayout.length() == 0 ? "p:64:64" : "-p:64:64");
         else if (Src->getPointerSize() == Module::Pointer32)
           DataLayout.append(DataLayout.length() == 0 ? "p:32:32" : "-p:32:32");
+      }
       Dest->setDataLayout(DataLayout);
     }
   }
 
-  // COpy the target triple from the source to dest if the dest's is empty
+  // Copy the target triple from the source to dest if the dest's is empty.
   if (Dest->getTargetTriple().empty() && !Src->getTargetTriple().empty())
     Dest->setTargetTriple(Src->getTargetTriple());
       
@@ -976,7 +1140,7 @@ Linker::LinkModules(Module *Dest, Module *Src, std::string *ErrorMsg) {
       Dest->getTargetTriple() != Src->getTargetTriple())
     cerr << "WARNING: Linking two modules of different target triples!\n";
 
-  // Append the module inline asm string
+  // Append the module inline asm string.
   if (!Src->getModuleInlineAsm().empty()) {
     if (Dest->getModuleInlineAsm().empty())
       Dest->setModuleInlineAsm(Src->getModuleInlineAsm());
@@ -988,12 +1152,9 @@ Linker::LinkModules(Module *Dest, Module *Src, std::string *ErrorMsg) {
   // Update the destination module's dependent libraries list with the libraries
   // from the source module. There's no opportunity for duplicates here as the
   // Module ensures that duplicate insertions are discarded.
-  Module::lib_iterator SI = Src->lib_begin();
-  Module::lib_iterator SE = Src->lib_end();
-  while ( SI != SE ) {
+  for (Module::lib_iterator SI = Src->lib_begin(), SE = Src->lib_end();
+       SI != SE; ++SI) 
     Dest->addLibrary(*SI);
-    ++SI;
-  }
 
   // LinkTypes - Go through the symbol table of the Src module and see if any
   // types are named in the src module that are not named in the Dst module.
@@ -1030,6 +1191,11 @@ Linker::LinkModules(Module *Dest, Module *Src, std::string *ErrorMsg) {
   if (LinkFunctionProtos(Dest, Src, ValueMap, ErrorMsg))
     return true;
 
+  // If there were any alias, link them now. We really need to do this now,
+  // because all of the aliases that may be referenced need to be available in
+  // ValueMap
+  if (LinkAlias(Dest, Src, ValueMap, ErrorMsg)) return true;
+
   // Update the initializers in the Dest module now that all globals that may
   // be referenced are in Dest.
   if (LinkGlobalInits(Dest, Src, ValueMap, ErrorMsg)) return true;
@@ -1042,8 +1208,8 @@ Linker::LinkModules(Module *Dest, Module *Src, std::string *ErrorMsg) {
   // If there were any appending global variables, link them together now.
   if (LinkAppendingVars(Dest, AppendingVars, ErrorMsg)) return true;
 
-  // If there were any alias, link them now.
-  if (LinkAlias(Dest, Src, ErrorMsg)) return true;
+  // Resolve all uses of aliases with aliasees
+  if (ResolveAliases(Dest)) return true;
 
   // If the source library's module id is in the dependent library list of the
   // destination library, remove it since that module is now linked in.
