@@ -13,6 +13,7 @@
 //===----------------------------------------------------------------------===//
 
 #include "llvm/ADT/APFloat.h"
+#include "llvm/ADT/FoldingSet.h"
 #include <cassert>
 #include <cstring>
 #include "llvm/Support/MathExtras.h"
@@ -77,20 +78,20 @@ namespace llvm {
 /* Put a bunch of private, handy routines in an anonymous namespace.  */
 namespace {
 
-  inline unsigned int
+  static inline unsigned int
   partCountForBits(unsigned int bits)
   {
     return ((bits) + integerPartWidth - 1) / integerPartWidth;
   }
 
   /* Returns 0U-9U.  Return values >= 10U are not digits.  */
-  inline unsigned int
+  static inline unsigned int
   decDigitValue(unsigned int c)
   {
     return c - '0';
   }
 
-  unsigned int
+  static unsigned int
   hexDigitValue(unsigned int c)
   {
     unsigned int r;
@@ -110,7 +111,7 @@ namespace {
     return -1U;
   }
 
-  inline void
+  static inline void
   assertArithmeticOK(const llvm::fltSemantics &semantics) {
     assert(semantics.arithmeticOK
            && "Compile-time arithmetic does not support these semantics");
@@ -121,7 +122,7 @@ namespace {
 
      If the exponent overflows, returns a large exponent with the
      appropriate sign.  */
-  int
+  static int
   readExponent(const char *p)
   {
     bool isNegative;
@@ -159,12 +160,12 @@ namespace {
 
   /* This is ugly and needs cleaning up, but I don't immediately see
      how whilst remaining safe.  */
-  int
+  static int
   totalExponent(const char *p, int exponentAdjustment)
   {
-    integerPart unsignedExponent;
+    int unsignedExponent;
     bool negative, overflow;
-    long exponent;
+    int exponent;
 
     /* Move past the exponent letter and sign to the digits.  */
     p++;
@@ -205,7 +206,7 @@ namespace {
     return exponent;
   }
 
-  const char *
+  static const char *
   skipLeadingZeroesAndAnyDot(const char *p, const char **dot)
   {
     *dot = 0;
@@ -241,7 +242,7 @@ namespace {
     int normalizedExponent;
   };
 
-  void
+  static void
   interpretDecimal(const char *p, decimalInfo *D)
   {
     const char *dot;
@@ -279,9 +280,10 @@ namespace {
       while (*p == '.');
 
       /* Adjust the exponents for any decimal point.  */
-      D->exponent += (dot - p) - (dot > p);
-      D->normalizedExponent = (D->exponent + (p - D->firstSigDigit)
-                               - (dot > D->firstSigDigit && dot < p));
+      D->exponent += static_cast<exponent_t>((dot - p) - (dot > p));
+      D->normalizedExponent = (D->exponent +
+                static_cast<exponent_t>((p - D->firstSigDigit)
+                                        - (dot > D->firstSigDigit && dot < p)));
     }
 
     D->lastSigDigit = p;
@@ -290,7 +292,7 @@ namespace {
   /* Return the trailing fraction of a hexadecimal number.
      DIGITVALUE is the first hex digit of the fraction, P points to
      the next digit.  */
-  lostFraction
+  static lostFraction
   trailingHexadecimalFraction(const char *p, unsigned int digitValue)
   {
     unsigned int hexDigit;
@@ -318,7 +320,7 @@ namespace {
 
   /* Return the fraction lost were a bignum truncated losing the least
      significant BITS bits.  */
-  lostFraction
+  static lostFraction
   lostFractionThroughTruncation(const integerPart *parts,
                                 unsigned int partCount,
                                 unsigned int bits)
@@ -340,7 +342,7 @@ namespace {
   }
 
   /* Shift DST right BITS bits noting lost fraction.  */
-  lostFraction
+  static lostFraction
   shiftRight(integerPart *dst, unsigned int parts, unsigned int bits)
   {
     lostFraction lost_fraction;
@@ -353,7 +355,7 @@ namespace {
   }
 
   /* Combine the effect of two lost fractions.  */
-  lostFraction
+  static lostFraction
   combineLostFractions(lostFraction moreSignificant,
                        lostFraction lessSignificant)
   {
@@ -374,7 +376,7 @@ namespace {
 
      See "How to Read Floating Point Numbers Accurately" by William D
      Clinger.  */
-  unsigned int
+  static unsigned int
   HUerrBound(bool inexactMultiply, unsigned int HUerr1, unsigned int HUerr2)
   {
     assert(HUerr1 < 2 || HUerr2 < 2 || (HUerr1 + HUerr2 < 8));
@@ -388,7 +390,7 @@ namespace {
   /* The number of ulps from the boundary (zero, or half if ISNEAREST)
      when the least significant BITS are truncated.  BITS cannot be
      zero.  */
-  integerPart
+  static integerPart
   ulpsFromBoundary(const integerPart *parts, unsigned int bits, bool isNearest)
   {
     unsigned int count, partBits;
@@ -433,7 +435,7 @@ namespace {
 
   /* Place pow(5, power) in DST, and return the number of parts used.
      DST must be at least one part larger than size of the answer.  */
-  unsigned int
+  static unsigned int
   powerOf5(integerPart *dst, unsigned int power)
   {
     static integerPart firstEightPowers[] = { 1, 5, 25, 125, 625, 3125,
@@ -504,7 +506,7 @@ namespace {
   /* Write out an integerPart in hexadecimal, starting with the most
      significant nibble.  Write out exactly COUNT hexdigits, return
      COUNT.  */
-  unsigned int
+  static unsigned int
   partAsHex (char *dst, integerPart part, unsigned int count,
              const char *hexDigitChars)
   {
@@ -522,7 +524,7 @@ namespace {
   }
 
   /* Write out an unsigned decimal integer.  */
-  char *
+  static char *
   writeUnsignedDecimal (char *dst, unsigned int n)
   {
     char buff[40], *p;
@@ -540,7 +542,7 @@ namespace {
   }
 
   /* Write out a signed decimal integer.  */
-  char *
+  static char *
   writeSignedDecimal (char *dst, int value)
   {
     if (value < 0) {
@@ -627,14 +629,14 @@ APFloat::bitwiseIsEqual(const APFloat &rhs) const {
       category != rhs.category ||
       sign != rhs.sign)
     return false;
-  if (semantics==(const llvm::fltSemantics* const)&PPCDoubleDouble &&
+  if (semantics==(const llvm::fltSemantics*)&PPCDoubleDouble &&
       sign2 != rhs.sign2)
     return false;
   if (category==fcZero || category==fcInfinity)
     return true;
   else if (category==fcNormal && exponent!=rhs.exponent)
     return false;
-  else if (semantics==(const llvm::fltSemantics* const)&PPCDoubleDouble &&
+  else if (semantics==(const llvm::fltSemantics*)&PPCDoubleDouble &&
            exponent2!=rhs.exponent2)
     return false;
   else {
@@ -691,6 +693,11 @@ APFloat::~APFloat()
   freeSignificand();
 }
 
+// Profile - This method 'profiles' an APFloat for use with FoldingSet.
+void APFloat::Profile(FoldingSetNodeID& ID) const {
+  ID.Add(convertToAPInt());
+}
+
 unsigned int
 APFloat::partCount() const
 {
@@ -1672,6 +1679,7 @@ APFloat::convert(const fltSemantics &toSemantics,
   opStatus fs;
 
   assertArithmeticOK(*semantics);
+  assertArithmeticOK(toSemantics);
   lostFraction = lfExactlyZero;
   newPartCount = partCountForBits(toSemantics.precision + 1);
   oldPartCount = partCount();
@@ -1712,6 +1720,8 @@ APFloat::convert(const fltSemantics &toSemantics,
     fs = normalize(rounding_mode, lostFraction);
   } else if (category == fcNaN) {
     int shift = toSemantics.precision - semantics->precision;
+    // Do this now so significandParts gets the right answer
+    semantics = &toSemantics;
     // No normalization here, just truncate
     if (shift>0)
       APInt::tcShiftLeft(significandParts(), newPartCount, shift);
@@ -1721,7 +1731,6 @@ APFloat::convert(const fltSemantics &toSemantics,
     // does not give you back the same bits.  This is dubious, and we
     // don't currently do it.  You're really supposed to get
     // an invalid operation signal at runtime, but nobody does that.
-    semantics = &toSemantics;
     fs = opOK;
   } else {
     semantics = &toSemantics;
@@ -1906,6 +1915,23 @@ APFloat::convertFromUnsignedParts(const integerPart *src,
   return normalize(rounding_mode, lost_fraction);
 }
 
+APFloat::opStatus
+APFloat::convertFromAPInt(const APInt &Val,
+                          bool isSigned,
+                          roundingMode rounding_mode)
+{
+  unsigned int partCount = Val.getNumWords();
+  APInt api = Val;
+
+  sign = false;
+  if (isSigned && api.isNegative()) {
+    sign = true;
+    api = -api;
+  }
+
+  return convertFromUnsignedParts(api.getRawData(), partCount, rounding_mode);
+}
+
 /* Convert a two's complement integer SRC to a floating point number,
    rounding according to ROUNDING_MODE.  ISSIGNED is true if the
    integer is signed, in which case it must be sign-extended.  */
@@ -1977,7 +2003,7 @@ APFloat::convertFromHexadecimalString(const char *p,
   firstSignificantDigit = p;
 
   for(;;) {
-    integerPart hex_value;
+    unsigned int hex_value;
 
     if(*p == '.') {
       assert(dot == 0);
@@ -2018,7 +2044,7 @@ APFloat::convertFromHexadecimalString(const char *p,
 
     /* Calculate the exponent adjustment implicit in the number of
        significant digits.  */
-    expAdjustment = dot - firstSignificantDigit;
+    expAdjustment = static_cast<int>(dot - firstSignificantDigit);
     if(expAdjustment < 0)
       expAdjustment++;
     expAdjustment = expAdjustment * 4 - 1;
@@ -2072,7 +2098,8 @@ APFloat::roundSignificandWithExponent(const integerPart *decSigParts,
     decSig.exponent += exp;
 
     lostFraction calcLostFraction;
-    integerPart HUerr, HUdistance, powHUerr;
+    integerPart HUerr, HUdistance;
+    unsigned int powHUerr;
 
     if (exp >= 0) {
       /* multiplySignificand leaves the precision-th bit set to 1.  */
@@ -2088,7 +2115,7 @@ APFloat::roundSignificandWithExponent(const integerPart *decSigParts,
           excessPrecision = calcSemantics.precision;
       }
       /* Extra half-ulp lost in reciprocal of exponent.  */
-      powHUerr = (powStatus == opOK && calcLostFraction == lfExactlyZero) ? 0: 2;
+      powHUerr = (powStatus == opOK && calcLostFraction == lfExactlyZero) ? 0:2;
     }
 
     /* Both multiplySignificand and divideSignificand return the
@@ -2165,7 +2192,7 @@ APFloat::convertFromDecimalString(const char *p, roundingMode rounding_mode)
        N-digit decimal integer is N * 196 / 59.  Allocate enough space
        to hold the full significand, and an extra part required by
        tcMultiplyPart.  */
-    partCount = (D.lastSigDigit - D.firstSigDigit) + 1;
+    partCount = static_cast<unsigned int>(D.lastSigDigit - D.firstSigDigit) + 1;
     partCount = partCountForBits(1 + 196 * partCount / 59);
     decSignificand = new integerPart[partCount + 1];
     partCount = 0;
@@ -2295,7 +2322,7 @@ APFloat::convertToHexString(char *dst, unsigned int hexDigits,
 
   *dst = 0;
 
-  return dst - p;
+  return static_cast<unsigned int>(dst - p);
 }
 
 /* Does the hard work of outputting the correctly rounded hexadecimal
@@ -2418,7 +2445,7 @@ APFloat::getHashValue() const
     uint32_t hash = sign<<11 | semantics->precision | exponent<<12;
     const integerPart* p = significandParts();
     for (int i=partCount(); i>0; i--, p++)
-      hash ^= ((uint32_t)*p) ^ (*p)>>32;
+      hash ^= ((uint32_t)*p) ^ (uint32_t)((*p)>>32);
     return hash;
   }
 }
@@ -2435,7 +2462,7 @@ APFloat::getHashValue() const
 APInt
 APFloat::convertF80LongDoubleAPFloatToAPInt() const
 {
-  assert(semantics == (const llvm::fltSemantics* const)&x87DoubleExtended);
+  assert(semantics == (const llvm::fltSemantics*)&x87DoubleExtended);
   assert (partCount()==2);
 
   uint64_t myexponent, mysignificand;
@@ -2458,8 +2485,8 @@ APFloat::convertF80LongDoubleAPFloatToAPInt() const
   }
 
   uint64_t words[2];
-  words[0] =  (((uint64_t)sign & 1) << 63) |
-              ((myexponent & 0x7fff) <<  48) |
+  words[0] =  ((uint64_t)(sign & 1) << 63) |
+              ((myexponent & 0x7fffLL) <<  48) |
               ((mysignificand >>16) & 0xffffffffffffLL);
   words[1] = mysignificand & 0xffff;
   return APInt(80, 2, words);
@@ -2468,7 +2495,7 @@ APFloat::convertF80LongDoubleAPFloatToAPInt() const
 APInt
 APFloat::convertPPCDoubleDoubleAPFloatToAPInt() const
 {
-  assert(semantics == (const llvm::fltSemantics* const)&PPCDoubleDouble);
+  assert(semantics == (const llvm::fltSemantics*)&PPCDoubleDouble);
   assert (partCount()==2);
 
   uint64_t myexponent, mysignificand, myexponent2, mysignificand2;
@@ -2501,10 +2528,10 @@ APFloat::convertPPCDoubleDoubleAPFloatToAPInt() const
   }
 
   uint64_t words[2];
-  words[0] =  (((uint64_t)sign & 1) << 63) |
+  words[0] =  ((uint64_t)(sign & 1) << 63) |
               ((myexponent & 0x7ff) <<  52) |
               (mysignificand & 0xfffffffffffffLL);
-  words[1] =  (((uint64_t)sign2 & 1) << 63) |
+  words[1] =  ((uint64_t)(sign2 & 1) << 63) |
               ((myexponent2 & 0x7ff) <<  52) |
               (mysignificand2 & 0xfffffffffffffLL);
   return APInt(128, 2, words);
@@ -2535,7 +2562,7 @@ APFloat::convertDoubleAPFloatToAPInt() const
     mysignificand = *significandParts();
   }
 
-  return APInt(64, (((((uint64_t)sign & 1) << 63) |
+  return APInt(64, ((((uint64_t)(sign & 1) << 63) |
                      ((myexponent & 0x7ff) <<  52) |
                      (mysignificand & 0xfffffffffffffLL))));
 }
@@ -2550,7 +2577,7 @@ APFloat::convertFloatAPFloatToAPInt() const
 
   if (category==fcNormal) {
     myexponent = exponent+127; //bias
-    mysignificand = *significandParts();
+    mysignificand = (uint32_t)*significandParts();
     if (myexponent == 1 && !(mysignificand & 0x800000))
       myexponent = 0;   // denormal
   } else if (category==fcZero) {
@@ -2562,7 +2589,7 @@ APFloat::convertFloatAPFloatToAPInt() const
   } else {
     assert(category == fcNaN && "Unknown category!");
     myexponent = 0xff;
-    mysignificand = *significandParts();
+    mysignificand = (uint32_t)*significandParts();
   }
 
   return APInt(32, (((sign&1) << 31) | ((myexponent&0xff) << 23) |
@@ -2576,16 +2603,16 @@ APFloat::convertFloatAPFloatToAPInt() const
 APInt
 APFloat::convertToAPInt() const
 {
-  if (semantics == (const llvm::fltSemantics* const)&IEEEsingle)
+  if (semantics == (const llvm::fltSemantics*)&IEEEsingle)
     return convertFloatAPFloatToAPInt();
   
-  if (semantics == (const llvm::fltSemantics* const)&IEEEdouble)
+  if (semantics == (const llvm::fltSemantics*)&IEEEdouble)
     return convertDoubleAPFloatToAPInt();
 
-  if (semantics == (const llvm::fltSemantics* const)&PPCDoubleDouble)
+  if (semantics == (const llvm::fltSemantics*)&PPCDoubleDouble)
     return convertPPCDoubleDoubleAPFloatToAPInt();
 
-  assert(semantics == (const llvm::fltSemantics* const)&x87DoubleExtended &&
+  assert(semantics == (const llvm::fltSemantics*)&x87DoubleExtended &&
          "unknown format!");
   return convertF80LongDoubleAPFloatToAPInt();
 }
@@ -2593,7 +2620,7 @@ APFloat::convertToAPInt() const
 float
 APFloat::convertToFloat() const
 {
-  assert(semantics == (const llvm::fltSemantics* const)&IEEEsingle);
+  assert(semantics == (const llvm::fltSemantics*)&IEEEsingle);
   APInt api = convertToAPInt();
   return api.bitsToFloat();
 }
@@ -2601,7 +2628,7 @@ APFloat::convertToFloat() const
 double
 APFloat::convertToDouble() const
 {
-  assert(semantics == (const llvm::fltSemantics* const)&IEEEdouble);
+  assert(semantics == (const llvm::fltSemantics*)&IEEEdouble);
   APInt api = convertToAPInt();
   return api.bitsToDouble();
 }
@@ -2624,7 +2651,7 @@ APFloat::initFromF80LongDoubleAPInt(const APInt &api)
   initialize(&APFloat::x87DoubleExtended);
   assert(partCount()==2);
 
-  sign = i1>>63;
+  sign = static_cast<unsigned int>(i1>>63);
   if (myexponent==0 && mysignificand==0) {
     // exponent, significand meaningless
     category = fcZero;
@@ -2660,8 +2687,8 @@ APFloat::initFromPPCDoubleDoubleAPInt(const APInt &api)
   initialize(&APFloat::PPCDoubleDouble);
   assert(partCount()==2);
 
-  sign = i1>>63;
-  sign2 = i2>>63;
+  sign = static_cast<unsigned int>(i1>>63);
+  sign2 = static_cast<unsigned int>(i2>>63);
   if (myexponent==0 && mysignificand==0) {
     // exponent, significand meaningless
     // exponent2 and significand2 are required to be 0; we don't check
@@ -2707,7 +2734,7 @@ APFloat::initFromDoubleAPInt(const APInt &api)
   initialize(&APFloat::IEEEdouble);
   assert(partCount()==1);
 
-  sign = i>>63;
+  sign = static_cast<unsigned int>(i>>63);
   if (myexponent==0 && mysignificand==0) {
     // exponent, significand meaningless
     category = fcZero;
