@@ -12,7 +12,6 @@
 //===----------------------------------------------------------------------===//
 
 #include "llvm/Support/Dwarf.h"
-#include "llvm/System/IncludeFile.h"
 
 #include <cassert>
 
@@ -582,5 +581,3 @@ const char *CallFrameString(unsigned Encoding) {
 } // End of namespace dwarf.
 
 } // End of namespace llvm.
-
-DEFINING_FILE_FOR(SupportDwarf)
