@@ -14,7 +14,6 @@
 
 #include "llvm/System/Memory.h"
 #include "llvm/Config/config.h"
-#include "llvm/System/IncludeFile.h"
 
 namespace llvm {
 using namespace sys;
@@ -33,5 +32,3 @@ using namespace sys;
 #ifdef LLVM_ON_WIN32
 #include "Win32/Memory.inc"
 #endif
-
-DEFINING_FILE_FOR(SystemMemory)
