@@ -101,7 +101,6 @@
 #include "llvm/Transforms/Utils/Local.h"
 #include <algorithm>
 #include <deque>
-#include <sstream>
 #include <stack>
 using namespace llvm;
 
@@ -244,6 +243,7 @@ namespace {
              *Node2 = getNodeForBlock(BB2);
         return Node1 && Node2 && Node1->dominates(Node2);
       }
+      return false; // Not reached
     }
 
   private:
@@ -717,7 +717,7 @@ namespace {
 
           if (edge.LV == J->LV)
             return; // This update adds nothing new.
-	}
+        }
 
         if (I != B) {
           // We also have to tighten any edge beneath our update.
@@ -729,7 +729,7 @@ namespace {
             }
             if (K == B) break;
           }
-	}
+        }
 
         // Insert new edge at Subtree if it isn't already there.
         if (I == E || I->To != n || Subtree != I->Subtree)
@@ -1112,7 +1112,7 @@ namespace {
       else if (isa<ConstantPointerNull>(V))
         return ConstantRange(APInt::getNullValue(typeToWidth(V->getType())));
       else
-        return typeToWidth(V->getType());
+        return ConstantRange(typeToWidth(V->getType()));
     }
 
     // typeToWidth - returns the number of bits necessary to store a value of
@@ -1413,6 +1413,7 @@ namespace {
         if (!Node) return false;
         return Top->dominates(Node);
       }
+      return false; // Not reached
     }
 
     // aboveOrBelow - true if the Instruction either dominates or is dominated
